@@ -1402,7 +1402,7 @@ def test_script_defaults():
 
 
 @pytest.mark.asyncio
-async def test_script_streams_continiously(capsys):
+async def test_script_streams_continuously(capsys):
     ip = get_ipython()
     # Windows is slow to start up a thread on CI
     is_windows = os.name == "nt"
@@ -1424,7 +1424,7 @@ async def test_script_streams_continiously(capsys):
     thread = Thread(target=print_numbers)
     thread.start()
     sleep(step / 2)
-    ip.run_cell_magic("script", f"{sys.executable}", code)
+    ip.run_cell_magic("script", sys.executable, code)
     thread.join()
 
     captured = capsys.readouterr()
@@ -1433,6 +1433,21 @@ async def test_script_streams_continiously(capsys):
     assert captured.out == "0.1.2."
 
 
+def test_script_streams_multibyte_unicode(capsys):
+    ip = get_ipython()
+    # € in UTF-8 is encoded using 3 bytes
+    code = "print('€' * 1000, end='')"
+    is_windows = os.name == "nt"
+    command = sys.executable
+    if is_windows:
+        # windows does not use UTF for streams/pipes by default
+        command += " -X utf8"
+    ip.run_cell_magic("script", command, code)
+
+    captured = capsys.readouterr()
+    assert captured.out == "€" * 1000
+
+
 @magics_class
 class FooFoo(Magics):
     """class with both %foo and %%foo magics"""
